#!/usr/bin/perl -w

# $Id: dr_no-reply-hook.pl,v 1.2 2006/10/06 13:09:20 semik Exp $
#
# dr_no-reply-hook.pl is plugin to implementing dead-realm feature for
# Radiator as replacement for dead-host feature which is not suitable
# for proxy networks such as eduroam.
#
# Author: Jan Tomek <jan.tomasek@cesnet.cz>, <jan@tomasek.cz>
# Copyright (c) 2006, CESNET, z.s.p.o.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.
#
# See README and COPYING for more details.


# Maximum number of dead realm records which will be kept in cache for
# each defined host. Actualy number of kept records will vary from
# max*0.9 to max.
#
# For 10000 records is needed 11384-9508=1876kB memory (length of
# realm was "test09999.orgA.etest.cesnet.cz").
#
my $max_dead_realm_records = 1000;

use Data::Dumper;

sub semik_get_realm {
  my $username = shift;

  return $1 if ($username =~ /@(.+)$/);
  return undef;
}

sub {
  my $username = ${$_[1]}->get_attr('User-Name');
  my $hostname = ${$_[1]}->{ThisHost}->{Name};
  my $realm = semik_get_realm($username) || 'unknown-realm';

  my $dead_realm_records = scalar(keys %{$main::deadRealms->{$hostname}});
  if ($dead_realm_records > $max_dead_realm_records) {
    # Number of dead realm record exceeded
    # $max_dead_realm_records. Someone is propably trying to fill our
    # memory by trash. We have to delete oldest records.
    foreach my $realm (sort {$main::deadRealms->{$hostname}->{$a} <=> $main::deadRealms->{$hostname}->{$b}} keys %{$main::deadRealms->{$hostname}}) {
      &main::log($main::LOG_DEBUG, "dead-realm: removing realm \@$realm from host $hostname cache ($dead_realm_records).");
      delete $main::deadRealms->{$hostname}->{$realm};
      $dead_realm_records--;
      last if ($dead_realm_records < ($max_dead_realm_records*0.9));
    };
  };

  # Mark this host dead for this realm
  $main::deadRealms->{$hostname}->{$realm} = time;
  &main::log($main::LOG_DEBUG, "dead-realm: Marking realm \@$realm dead on host $hostname cache ($dead_realm_records).");
};
